from datetime import datetime
from django.db import models
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TextModel, SDModel


class DateModel(SDModel):
    date = models.DateTimeField()


class DateModelSerializer(ModelSerializer):
    class Meta:
        model = DateModel
        publish_fields = ('date')
        update_fields = ('date')


class TextModelSerializer(ModelSerializer):
    class Meta:
        model = TextModel
        publish_fields = ('text')
        update_fields = ('text')


class TestModelSerializer(DragonTestCase):
    def test_deserialize_model(self):
        data = {'text': 'foo'}
        serializer = TextModelSerializer(data)
        model_instance = serializer.save()
        self.assertEqual(model_instance.text, data['text'])

    def test_passing_invalid_data(self):
        foo = 'text'
        with self.assertRaises(Exception):
            TextModelSerializer(foo)

    def test_ignore_non_model_fields(self):
        data = {'text': 'foo', 'random_field': 'val'}
        serializer = TextModelSerializer(data)
        model_instance = serializer.deserialize()
        self.assertEqual(model_instance.text, data['text'])

    def test_deserialize_field(self):
        date = datetime.now()
        data = {'date': str(date)}
        serializer = DateModelSerializer(data)
        object = serializer.save()
        self.assertEqual(object.date, date)



from ..route_handler import BaseModelRouter, SUCCESS, ERROR
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelRouter):
    model = TwoFieldModel
    serializer_class = Serializer


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)

    def test_successful_create(self):
        data = {'text': 'text', 'number': 3}
        self.router.create(**data)
        model = self.router.model.objects.get()
        self.assertIsNotNone(model)

    def test_error_on_create(self):
        data = {'text': 'text'}
        self.router.create(**data)
        actual = self.connection.last_message
        self.assertEqual(actual['context']['state'], ERROR)
        self.assertIn('number', actual['data'])

    def test_created(self):
        data = {'text': 'text', 'number': 3}
        self.router.create(**data)
        model = self.router.model.objects.get()
        actual = self.connection.last_message
        self.assertEqual(actual['context']['state'], SUCCESS)
        self.assertDictEqual(actual['data'], Serializer(instance=model).serialize())



from ..route_handler import BaseModelRouter, SUCCESS, ERROR
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        publish_fields = ('text', 'number')
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelRouter):
    model = TwoFieldModel
    serializer_class = Serializer

    def get_object(self, **kwargs):
        return self.model.objects.get(pk=kwargs['id'])


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)
        self.obj = self.router.model.objects.create(text='text', number=1)

    def test_successful_update(self):
        data = {'number': 3, 'id': self.obj.pk}
        self.router.update(**data)
        self.assertEqual(3, self.router.model.objects.get(pk=self.obj.pk).number)

    def test_error_on_update(self):
        data = {'text': None, 'id': self.obj.pk}
        self.router.update(**data)
        actual = self.connection.last_message
        self.assertEqual(actual['context']['state'], ERROR)
        self.assertIn('text', actual['data'])

    def test_updated(self):
        data = {'text': 'text', 'number': 3, 'id': self.obj.pk}
        self.router.update(**data)
        model = self.router.model.objects.get()
        actual = self.connection.last_message
        self.assertEqual(actual['context']['state'], SUCCESS)
        self.assertIn('number', actual['data'])
        self.assertNotIn('text', actual['data'])



from ..route_handler import BaseRouter
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    valid_verbs = ('do_something', )
    route_name = 'foo'

    def get_client_context(self, verb, **kwargs):
        context = super(FooRouter, self).get_client_context(verb, **kwargs)
        context['extra'] = 'value'
        return context

    def do_something(self, **kwargs):
        self.send({'foo': 'bar'})


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_get_client_context(self):
        data = {'verb': 'do_something'}
        self.router.handle(data)

        message = self.connection.last_message
        client_context = message['context']['client_context']
        self.assertDictEqual(client_context, self.router.get_client_context('do_something'))

    def test_empty_client_context(self):
        self.assertIsNone(self.router._update_client_context(None))



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = BaseModelRouter(self.connection)

    def test_on_error(self):
        self.router.on_error({'key': 'error'})
        actual = self.connection.last_message
        expected = {'context': {'state': 'error'}, 'data': {'key': 'error'}}
        self.assertDictEqual(actual, expected)



from ..serializers.serializer_tools import get_serializer_relationship_field
from .channel_utils import make_safe, get_property_and_value_from_channel, properties_match_channel_by_object, properties_match_channel_by_dict


def _construct_channel(base_channel, **channel_filter):
    sorted_filter_keys = sorted(channel_filter)
    filter_string = '|'.join(['{}:{}'.format(k, make_safe(channel_filter[k])) for k in sorted_filter_keys])
    complete_channel = '{}{}'.format(base_channel, filter_string)
    return complete_channel


def _prefix_channel_filter(var_name, channel_filter):
    """
    Prefix the property with the related var name
    """
    return channel_filter.replace('|', '|{}__'.format(var_name))


def make_channels(serializer, related_serializers=None, property_filter=None, prefix=None):
    channels = []
    base_channel = serializer.get_base_channel()

    if property_filter:
        if not isinstance(property_filter, list):
            property_filter = [property_filter]
        for p in property_filter:
            channel_data = _construct_channel(base_channel, **p)
            if prefix:
                channel_data = _prefix_channel_filter(prefix, channel_data)
            channels.append(channel_data)
    else:
        channels.append(_construct_channel(base_channel))

    if related_serializers:
        for related_serializer in related_serializers:
            field_name = get_serializer_relationship_field(serializer, related_serializer)
            channels += make_channels(related_serializer, None, property_filter, field_name)
    return channels


def filter_channels_by_dict(channels, dict):
    result = []
    for channel in channels:
        channel_properties = get_property_and_value_from_channel(channel)
        if not channel_properties:
            result.append(channel)
            continue
        if properties_match_channel_by_dict(dict, channel_properties):
            result.append(channel)
    return result


def filter_channels_by_model(channels, obj):
    result = []
    for channel in channels:
        channel_properties = get_property_and_value_from_channel(channel)
        if not channel_properties:
            result.append(channel)
            continue
        if properties_match_channel_by_object(obj, channel_properties):
            result.append(channel)
    return result


def has_related_values(obj, properties):
    for field, channel_val in properties:
        if '__' not in field:
            filter_name = channel_val
            property_name = field
        else:
            property_name, filter_name = field.split('__', 1)
        attr = getattr(obj, property_name)
        if hasattr(attr, 'all'):
            if not getattr(obj, property_name).filter(**{filter_name: channel_val}).exists():
                return False
    return True



from django.test import TestCase
from mimetypes import MimeTypes
from swampdragon.tests.dragon_test_case import DragonTestCase


# class DragonDjangoTestCase(DragonTestCase, TestCase):
#     pass


class FileUploadRequestData(object):
    def __init__(self, files):
        if not isinstance(files, list):
            files = [files]
        self.boundary = '--dragontestclient'
        self.body = ''
        for f in files:
            self.add_file(f)
        self.close_body()

    def get_headers(self):
        return {
            'Accept-Language': 'en-US,en;q=0.8',
            'Content-Length': '{}'.format(len(self.get_body())),
            'Origin': '/',
            'Content-Type': 'multipart/form-data; boundary={}'.format(self.boundary),
        }

    def add_file(self, file):
        mime = MimeTypes()
        mime_type = mime.guess_type(file.name)[0]
        data = {
            'file': file,
            'boundary': self.boundary,
            'mimetype': mime_type,
            'filename': file.name.split('/')[-1],
            'file_body': file.read().decode()
        }
        self.body += '--{boundary}\r\n\r\n'.format(**data)
        self.body += 'Content-Disposition: form-data; name="uploadedFile"; filename="{filename}"\r\n'.format(**data)
        self.body += 'Content-Type: {mimetype}\r\n\r\n'.format(**data)
        self.body += '{file_body}'.format(**data)
        self.body += '\r\n'

    def close_body(self):
        self.body += '--{}--\r\n\r\n'.format(self.boundary)

    def get_body(self):
        return self.body



from ..route_handler import BaseModelRouter, SUCCESS
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelRouter):
    model = TwoFieldModel
    serializer_class = Serializer


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)

    def test_subscribe(self):
        data = {'channel': 'client-channel'}
        self.router.subscribe(**data)
        self.assertEqual(self.connection.last_message['context']['state'], SUCCESS)
        self.assertIn('channel_data', self.connection.last_message)
        self.assertEqual(self.connection.last_message['channel_data']['local_channel'], 'client-channel')



from ..route_handler import BaseModelPublisherRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from ..pubsub_providers.base_provider import PUBACTIONS
from .models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        publish_fields = ('text', 'number')
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelPublisherRouter):
    model = TwoFieldModel
    serializer_class = Serializer

    def get_object(self, **kwargs):
        return self.model.objects.get(pk=kwargs['id'])

    def get_subscription_contexts(self, **kwargs):
        if 'number' in kwargs:
            return {'number': kwargs['number']}
        return {}


class TestBaseModelPublisherRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)
        self.obj = self.router.model.objects.create(text='text', number=123)

    def test_updated(self):
        data = {'text': 'text', 'number': 3, 'id': self.obj.pk}
        self.router.update(**data)
        model = self.router.model.objects.get()
        self.assertEqual(model.number, data['number'])

    def test_updated_publish(self):
        self.router.subscribe(**{'channel': 'client-channel'})
        data = {'text': 'text', 'number': 3, 'id': self.obj.pk}
        self.router.update(**data)
        actual = self.connection.get_last_published()
        expected = {'action': 'updated', 'channel': 'twofieldmodel|', 'data': {'_type': 'twofieldmodel', 'id': 1, 'number': 3}}
        self.assertDictEqual(actual, expected)

    def test_updated_remove_from_channel(self):
        """
        When a model no longer match the channel the subscriber should
        receive a DELETE action
        """
        self.router.subscribe(**{'channel': 'client-channel', 'number': 2})
        data = {'text': 'text', 'number': 2, 'id': self.obj.pk}
        self.router.update(**data)
        self.assertEqual(self.connection.last_pub['action'], PUBACTIONS.updated)

        data = {'text': 'text', 'number': 200, 'id': self.obj.pk}
        self.router.update(**data)
        self.assertEqual(self.connection.last_pub['action'], PUBACTIONS.deleted)




def login_required(func):
    def not_logged_in(self, **kwargs):
        self.send_login_required({'signin_required': 'you need to sign in'})
        return

    def check_user(self, **kwargs):
        user = self.connection.user
        if not user:
            return not_logged_in(self, **kwargs)
        return func(self, **kwargs)
    return check_user


class RoutePermission(object):
    def test_permission(self, handler, verb, **kwargs):
        raise NotImplementedError("You need to implement test_permission")

    def permission_failed(self, handler):
        raise NotImplementedError("You need to implement permission_failed")


class LoginRequired(RoutePermission):
    def __init__(self, verbs=None):
        self.test_against_verbs = verbs

    def test_permission(self, handler, verb, **kwargs):
        if not self.test_against_verbs:
            return handler.connection.user is not None
        if self.test_against_verbs:
            if verb not in self.test_against_verbs:
                return True
        user = handler.connection.user
        return user is not None

    def permission_failed(self, handler):
        handler.send_login_required()



from .session_store import BaseSessionStore

sessions = {}


class MockSessionStore(BaseSessionStore):
    def save(self, key, val):
        sessions[key] = val

    def get(self, key):
        try:
            return sessions[key]
        except:
            return None



from ..route_handler import BaseRouter
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    pass


class TestBaseRouter(DragonTestCase):
    def test_get_list(self):
        channel_data = FooRouter(self.connection).make_channel_data('client_chan', 'server_chan')
        expected = {'local_channel': 'client_chan', 'remote_channels': 'server_chan'}
        self.assertDictEqual(channel_data, expected)



import uuid


class BaseSessionStore(object):
    def save(self, key, val):
        raise NotImplemented()

    def save_get_key(self, val):
        key = self.generate_key()
        self.save(key, val)
        return key

    def get(self, key):
        raise NotImplemented()

    def generate_key(self):
        key = uuid.uuid4()
        return key.hex

    def refresh_key_timeout(self, key):
        raise NotImplemented()



from datetime import datetime
from ..route_handler import ModelRouter
from ..pubsub_providers.base_provider import PUBACTIONS
from .dragon_test_case import DragonTestCase
from .models import FooSelfPub, BarSelfPub
from .serializers import FooSelfPubSerializer, BarSelfPubSerializer


class FooModelRouter(ModelRouter):
    serializer_class = FooSelfPubSerializer


class BarModelRouter(ModelRouter):
    serializer_class = BarSelfPubSerializer


class TestSelfPubModel(DragonTestCase):
    def test_self_pub_model(self):
        router = FooModelRouter(self.connection)
        router.subscribe(**{'channel': 'testchan'})
        self.assertIsNone(self.connection.last_pub)
        FooSelfPub.objects.create(name='test')
        self.assertIsNotNone(self.connection.last_pub)

    def test_self_pub_model_with_fk(self):
        router = BarModelRouter(self.connection)
        router.subscribe(**{'channel': 'testchan'})
        self.assertIsNone(self.connection.last_pub)
        foo = FooSelfPub.objects.create(name='test')
        BarSelfPub.objects.create(date=datetime.now(), foo=foo)
        self.assertIsNotNone(self.connection.last_pub)

    def test_ignore_id_when_getting_updated_fields(self):
        FooSelfPubSerializer.Meta.publish_fields += ('pk', )
        foo = FooSelfPub.objects.create(name='test')

    def test_get_changes(self):
        foo = FooSelfPub.objects.create(name='test')
        self.assertListEqual(foo.get_changed_fields(), [])
        foo.number = 12
        self.assertListEqual(foo.get_changed_fields(), ['number'])
        foo.name = 'updated'
        self.assertIn('number', foo.get_changed_fields())
        self.assertIn('name', foo.get_changed_fields())

        bar = BarSelfPub.objects.create(date=datetime.now(), foo=foo)
        self.assertListEqual(bar.get_changed_fields(), [])
        update_date = datetime.now()
        bar.date = update_date
        self.assertListEqual(bar.get_changed_fields(), ['date'])

    def test_raise_validation_error(self):
        foo = FooSelfPub.objects.create(name='test')
        data = foo.serialize()
        self.assertEqual(data['name'], foo.name)

    def test_create(self):
        router = FooModelRouter(self.connection)
        router.subscribe(**{'channel': 'testchan'})
        FooSelfPub.objects.create(name='test')
        self.assertEqual(self.connection.last_pub['action'], 'created')

    def test_update(self):
        router = FooModelRouter(self.connection)
        router.subscribe(**{'channel': 'testchan'})
        foo = FooSelfPub.objects.create(name='test')
        foo.name = 'updated'
        foo.save()
        self.assertEqual(self.connection.last_pub['action'], 'updated')

    def test_remove_on_update(self):
        router = FooModelRouter(self.connection)
        router.subscribe(**{'channel': 'testchan', 'name__contains': 'findme'})
        foo = FooSelfPub.objects.create(name='test')
        self.assertIsNone(self.connection.last_pub)
        foo.name = 'findme'
        foo.save()
        self.assertEqual(self.connection.last_pub['action'], PUBACTIONS.updated)

        foo.name = 'hideme'
        foo.save()
        self.assertEqual(self.connection.last_pub['action'], PUBACTIONS.deleted)

        foo.name = 'findmeagain'
        foo.save()
        self.assertEqual(self.connection.last_pub['action'], PUBACTIONS.updated)



from ..serializers.model_serializer import ModelSerializer
from ..serializers.validation import ModelValidationError
from ..route_handler import ModelRouter, ERROR
from .. import route_handler
from .dragon_test_case import DragonTestCase
from .models import TwoFieldModel


class TwoFieldModelSerializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel
        update_fields = ('text', 'number')

    def validate_text(self, val):
        if val == 'foo':
            raise ModelValidationError({'text': ['text error']})

    def validate_number(self, val):
        if val == 8:
            raise ModelValidationError({'number': ['number error']})


class Router(ModelRouter):
    valid_verbs = ('create', )
    route_name = 'validation-router'
    serializer_class = TwoFieldModelSerializer


class TestSerializerValidation(DragonTestCase):
    def test_validate_field(self):
        data = {'text': 'foo', 'number': 8}
        ser = TwoFieldModelSerializer(data)
        with self.assertRaises(ModelValidationError):
            ser.save()

    def test_validate_field_from_router(self):
        """
        Ensure that both the text and number error message
        is raised upon validation
        """
        data = {'text': 'foo', 'number': 8}
        route_handler.register(Router)
        self.connection.call_verb(Router.route_name, 'create', **data)
        self.assertEqual(self.connection.last_message['context']['state'], ERROR)
        self.assertIn('text', self.connection.last_message['data'])
        self.assertIn('number', self.connection.last_message['data'])



from django.db import models
from ..models import SelfPublishModel
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import SDModel
from swampdragon import route_handler
from swampdragon.route_handler import ModelRouter


class Document(SelfPublishModel, SDModel):
    content = models.TextField()
    serializer_class = 'DocumentSerializer'


class Person(SelfPublishModel, SDModel):
    name = models.CharField(max_length=100)
    documents = models.ManyToManyField(Document)
    serializer_class = 'PersonSerializer'


class DocumentSerializer(ModelSerializer):
    person_set = 'PersonSerializer'

    class Meta:
        model = Document


class PersonSerializer(ModelSerializer):
    documents = DocumentSerializer

    class Meta:
        model = Person


class PersonRouter(ModelRouter):
    route_name = 'people'
    serializer_class = PersonSerializer
    include_related = [DocumentSerializer]

    def get_object(self, **kwargs):
        return Person.objects.get(pk=kwargs['id'])

    def get_query_set(self, **kwargs):
        return Person.objects.all()


class TestSelfPubModel(DragonTestCase):
    def setUp(self):
        route_handler.register(PersonRouter)

    def test_publish_m2m_on_add(self):
        self.connection.subscribe(PersonRouter.route_name, 'cli', {})
        for i in range(10):
            Person.objects.create(name='person {}'.format(i))
        doc = Document.objects.create(content='test', pk=99)
        self.connection.published_data = []
        doc.person_set = Person.objects.all()
        self.assertEqual(len(self.connection.published_data), 2)

    def test_publish_reverse_m2m_on_add(self):
        self.connection.subscribe(PersonRouter.route_name, 'cli', {})
        person_a = Person.objects.create(name='some person')
        person_b = Person.objects.create(name='another person')
        doc = Document.objects.create(content='test', pk=99)
        self.connection.published_data = []
        person_b.documents.add(doc)
        self.assertEqual(len(self.connection.published_data), 1)

    def test_publish_m2m_on_remove(self):
        self.connection.subscribe(PersonRouter.route_name, 'cli', {})
        person = Person.objects.create(name='a person')
        doc = Document.objects.create(content='test', pk=100)
        person.documents.add(doc)
        self.connection.published_data = []
        person.documents.remove(doc)
        lp = self.connection.last_pub
        self.assertEqual(lp['data']['documents'], [])



import json
import uuid
from .. import route_handler
from ..pubsub_providers.subscriber_factory import get_subscription_provider


pubsub = get_subscription_provider()


class TestConnection(object):
    uid = None
    channels = []

    def __init__(self, user=None):
        self.uid = str(uuid.uuid4())
        self.user = user

        self.sent_data = []
        self.published_data = []
        self.pub_sub = pubsub

    def send(self, message):
        self.sent_data.append(json.dumps(message))

    def publish(self, message):
        self.published_data.append(json.dumps(message))

    def client_send(self, data):
        if not isinstance(data, dict):
            data = json.loads(data)
        handler = route_handler.get_route_handler(data['route'])
        handler(self).handle(data)

    def call_verb(self, route, verb, **kwargs):
        self.client_send({
            'route': route,
            'verb': verb,
            'args': kwargs
        })
        return self.get_last_message()

    def get_user(self, **kwargs):
        return self.user

    def get_last_message(self):
        if not self.sent_data:
            return None
        last_message = self.sent_data[-1]
        if not isinstance(last_message, dict):
            last_message = json.loads(last_message)
        return last_message

    @property
    def last_message(self):
        return self.get_last_message()

    @property
    def last_pub(self):
        return self.get_last_published()

    def get_last_published(self):
        if not self.published_data:
            return None
        if isinstance(self.published_data[-1], dict):
            return self.published_data[-1]
        return json.loads(self.published_data[-1])

    def get_last_published_data(self):
        last_pub = self.get_last_published()
        if last_pub is None:
            return None
        return last_pub['data']

    def subscribe(self, route, client_channel, subscription_data):
        data = {'route': route, 'verb': 'subscribe', 'args': {'channel': client_channel}}
        data['args'].update(subscription_data)
        self.client_send(data)

    def unsubscribe(self, route, client_channel, subscription_data):
        data = {'route': route, 'verb': 'unsubscribe', 'args': {'channel': client_channel}}
        data['args'].update(subscription_data)
        self.client_send(data)

    def create(self, route, data):
        self.client_send({'route': route, 'verb': 'create', 'args': data})

    def update(self, route, data):
        self.client_send({'route': route, 'verb': 'update', 'args': data})

    def remove(self, route, data):
        self.client_send({'route': route, 'verb': 'delete', 'args': data})



#  Code from https://github.com/peterbe/django-sockjs-tornado
#  Tweaked to handle multiple routes
from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.importlib import import_module
from tornado import web, ioloop
from sockjs.tornado import SockJSRouter
from swampdragon import discover_routes, load_field_deserializers


class Command(BaseCommand):
    # option_list = BaseCommand.option_list + (
    #     make_option(
    #         '--host',
    #         action='store',
    #         dest='host',
    #         default='127.0.0.1',
    #         help='Host'),
    #     make_option(
    #         '--port',
    #         action='store',
    #         dest='port',
    #         default=getattr(settings, 'SOCKJS_PORT', 9999),
    #         help='What port number to run the socket server on'),
    #     make_option(
    #         '--no-keep-alive',
    #         action='store_true',
    #         dest='no_keep_alive',
    #         default=False,
    #         help='Set no_keep_alive on the connection if your server needs it')
    # )

    def handle(self, *args, **kwargs):
        HOST = '127.0.0.1'
        PORT = 9999

        if args:
            host_port = args[0]
            HOST = host_port.split(':')[0]
            PORT = host_port.split(':')[1]
        routers = []
        for sockjs_class in settings.SOCKJS_CLASSES:
            module_name, cls_name = sockjs_class[0].rsplit('.', 1)
            module = import_module(module_name)
            cls = getattr(module, cls_name)
            channel = sockjs_class[1]
            routers.append(SockJSRouter(cls, channel))
            print('Channel {}'.format(channel))

        app_settings = {
            'debug': settings.DEBUG,
        }

        urls = discover_routes()
        for router in routers:
            urls += router.urls

        load_field_deserializers()

        app = web.Application(urls, **app_settings)
        app.listen(PORT, address=HOST, no_keep_alive=False)
        print('Running sock app on {}:{}'.format(HOST, PORT))
        try:
            iol = ioloop.IOLoop.instance()
            # ioloop.IOLoop.set_blocking_log_threshold(iol, 1)
            iol.start()
        except KeyboardInterrupt:
            # so you don't think you erred when ^C'ing out
            pass



from django.db import models
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TextModel, SDModel


class Parent(SDModel):
    name = models.CharField(max_length=20)


class Child(SDModel):
    parent = models.ForeignKey(Parent)
    number = models.IntegerField()


class ChildSerializer(ModelSerializer):
    parent = 'ParentSerializer'

    class Meta:
        model = Child
        update_fields = ('number', 'parent')


class ParentSerializer(ModelSerializer):
    child_set = ChildSerializer

    class Meta:
        model = Parent
        publish_fields = ('name')
        update_fields = ('name', 'child_set')


class TestModelSerializer(DragonTestCase):
    def test_deserialize_with_fk(self):
        data = {
            'name': 'foo',
            'child_set': [
                {'number': 5}
            ]
        }
        serializer = ParentSerializer(data)
        parent = serializer.save()
        self.assertEqual(parent.name, data['name'])
        self.assertEqual(parent.child_set.first().number, data['child_set'][0]['number'])

    def test_deserialize_with_reverse_fk(self):
        data = {
            'number': 123,
            'parent': {
                'name': 'foo'
            }
        }
        serializer = ChildSerializer(data)
        child = serializer.save()
        self.assertEqual(child.number, data['number'])
        self.assertEqual(child.parent.name, data['parent']['name'])

    def test_get_object_map(self):
        """
        The object map should return information on how to map parents to children
        and vice versa
        """
        object_map = ParentSerializer.get_object_map()
        self.assertLessEqual(len(object_map), 2)
        for om in object_map:
            self.assertIn('via', om)



from ..route_handler import BaseRouter
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    route_name = 'foo'

    def get_subscription_channels(self, **kwargs):
        return ['foo-chan']


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_publish(self):
        data = {'channel': 'foo'}
        self.router.subscribe(**data)
        self.router.publish(['foo-chan'], {'key': 'value'})
        actual = self.connection.get_last_published()
        expected = {'key': 'value', 'channel': 'foo-chan'}
        self.assertDictEqual(actual, expected)



from .pubsub_providers.publisher_factory import get_publisher
from .paginator import Paginator
from .sessions.sessions import get_session_store
from .pubsub_providers.base_provider import PUBACTIONS
from .message_format import format_message
from .pubsub_providers.model_channel_builder import make_channels, filter_channels_by_model, filter_channels_by_dict
from .serializers.validation import ModelValidationError

SUCCESS = 'success'
ERROR = 'error'
LOGIN_REQUIRED = 'login_required'

registered_handlers = {}

publisher = get_publisher()


class UnexpectedVerbException(Exception):
    pass


class BaseRouter(object):
    valid_verbs = ['get_list', 'get_single', 'create', 'update', 'delete', 'subscribe', 'unsubscribe']
    exclude_verbs = []
    serializer = None
    route_name = None
    permission_classes = []
    session_store = get_session_store()

    def __init__(self, connection, request=None, **kwargs):
        self.connection = connection
        self.context = dict()

    def make_channel_data(self, client_channel, server_channels):
        return {'local_channel': client_channel, 'remote_channels': server_channels}

    @classmethod
    def get_name(cls):
        route_name = getattr(cls, 'route_name')
        if not route_name:
            raise Exception('\n------\n{} has no name.\nSet the route_name property\n------\n'.format(cls.__name__))
        return route_name

    def handle(self, data):
        verb = data['verb']
        kwargs = data.get('args', {})
        client_callback_name = data.get('callbackname')
        self.context['client_callback_name'] = client_callback_name
        self.context['verb'] = verb
        if '_page' in kwargs:
            self.context['page'] = kwargs.pop('_page')

        if verb in self.valid_verbs:
            m = getattr(self, verb)
            if self.permission_classes:
                for permission in self.permission_classes:
                    if not permission.test_permission(self, verb, **kwargs):
                        permission.permission_failed(self)
                        return
            m(**kwargs)
        else:
            if verb not in self.exclude_verbs:
                raise UnexpectedVerbException('\n------\nUnexpected verb: {}\n------'.format(verb))

    def get_client_context(self, verb, **kwargs):
        '''
        Additional data to be sent to the client with each `send` call
        from the router
        '''
        return {}

    def _update_client_context(self, data):
        if not data:
            return
        if 'client_context' not in self.context:
            self.context['client_context'] = {}
        self.context['client_context'].update(data)

    def get_list(self, **kwargs):
        raise NotImplementedError('get_list is not implemented')

    def get_single(self, **kwargs):
        raise NotImplementedError('get_single is not implemented')

    def create(self, **kwargs):
        raise NotImplementedError('create is not implemented')

    def update(self, **kwargs):
        raise NotImplementedError('update is not implemented')

    def delete(self, **kwargs):
        raise NotImplementedError('delete is not implemented')

    def send(self, data, channel_setup=None, **kwargs):
        self.context['state'] = SUCCESS
        if 'verb' in self.context:
            client_context = self.get_client_context(self.context['verb'], **kwargs)
            self._update_client_context(client_context)

        message = format_message(data=data, context=self.context, channel_setup=channel_setup)
        self.connection.send(message)

    def send_error(self, data, channel_setup=None):
        self.context['state'] = ERROR
        self.connection.send(format_message(data=data, context=self.context, channel_setup=channel_setup))

    def send_login_required(self, channel_setup=None):
        self.context['state'] = LOGIN_REQUIRED
        self.connection.send(format_message(data=None, context=self.context, channel_setup=channel_setup))

    def get_subscription_channels(self, **kwargs):
        raise NotImplementedError()

    def subscribe(self, **kwargs):
        client_channel = kwargs.pop('channel')
        server_channels = self.get_subscription_channels(**kwargs)
        self.send(
            data='subscribed',
            channel_setup=self.make_channel_data(client_channel, server_channels),
            **kwargs)
        self.connection.pub_sub.subscribe(server_channels, self.connection)

    def unsubscribe(self, **kwargs):
        client_channel = kwargs.pop('channel')
        server_channels = self.get_subscription_channels(**kwargs)
        self.send(
            data='unsubscribed',
            channel_setup=self.make_channel_data(client_channel, server_channels),
            **kwargs
        )
        self.connection.pub_sub.unsubscribe(server_channels, self.connection)

    def publish(self, channels, publish_data):
        for channel in channels:
            publish_data['channel'] = channel
            publisher.publish(channel, publish_data)


class BaseModelRouter(BaseRouter):
    model = None
    serializer_class = None
    instance = None
    include_related = []
    paginate_by = None
    _query_set = None
    _obj = None

    def _get_changed_fields(self, current_state, past_state):
        '''
        Compare the previous data with the current data
        to get only the changed fields
        '''
        changed_fields = []
        for o in current_state:
            if past_state[o] != current_state[o]:
                changed_fields.append(o)
        return changed_fields

    def _get_object(self, **kwargs):
        if self._obj:
            return self._obj
        self._obj = self.get_object(**kwargs)
        return self._obj

    def _get_query_set(self, **kwargs):
        if self._query_set:
            return self._query_set
        self._query_set = self.get_query_set(**kwargs)
        return self._query_set

    def get_list(self, **kwargs):
        obj_list = self._get_query_set(**kwargs)
        if self.paginate_by:
            page_num = self.context.get('page', 1)
            page = Paginator(obj_list, self.paginate_by).page(page_num)
            self._update_client_context({'page': page.serialize()})
            obj_list = page.object_list

        self.send_list(obj_list, **kwargs)
        return obj_list

    def get_initial(self, verb, **kwargs):
        return dict()

    def get_subscription_contexts(self, **kwargs):
        return dict(kwargs)

    def send_list(self, object_list, **kwargs):
        self.send([self.serializer_class(instance=o).serialize() for o in object_list], **kwargs)

    def get_single(self, **kwargs):
        obj = self._get_object(**kwargs)
        self.send_single(obj, **kwargs)
        return obj

    def send_single(self, obj, **kwargs):
        self.serializer = self.serializer_class(instance=obj)
        self.send(self.serializer.serialize(), **kwargs)

    def on_error(self, errors):
        self.send_error(errors)

    def create(self, **kwargs):
        initial = self.get_initial('create', **kwargs)
        self.serializer = self.serializer_class(data=kwargs, initial=initial)
        try:
            obj = self.serializer.save()
        except ModelValidationError as error:
            self.on_error(error.get_error_dict())
            return

        obj.save()
        self.created(obj, **kwargs)

    def created(self, obj, **kwargs):
        self.send(self.serializer.serialize())

    def update(self, **kwargs):
        initial = self.get_initial('update', **kwargs)
        obj = self._get_object(**kwargs)
        self.serializer = self.serializer_class(instance=obj, data=kwargs, initial=initial)
        past_state = self.serializer.serialize()
        try:
            self.serializer.save()
        except ModelValidationError as error:
            errors = error.get_error_dict()
            self.on_error(errors)
            return
        updated_fields = self._get_changed_fields(self.serializer.serialize(), past_state)

        self.updated(self.serializer.instance, updated_fields=updated_fields, past_state=past_state)

    def updated(self, obj, **kwargs):
        updated_fields = kwargs.get('updated_fields')
        self.send(self.serializer.serialize(fields=updated_fields), **kwargs)

    def delete(self, **kwargs):
        obj = self._get_object(**kwargs)
        self.serializer = self.serializer_class(instance=obj, data=kwargs)
        obj_id = obj.pk
        self.deleted(obj, obj_id, **kwargs)
        obj.delete()

    def deleted(self, obj, obj_id, **kwargs):
        serialized_obj = self.serializer.serialize()
        self.send(serialized_obj, **kwargs)

    def subscribe(self, **kwargs):
        client_channel = kwargs.pop('channel')
        server_channels = make_channels(self.serializer_class, self.include_related, self.get_subscription_contexts(**kwargs))
        data = self.serializer_class.get_object_map(self.include_related)
        channel_setup = self.make_channel_data(client_channel, server_channels)
        self.send(
            data=data,
            channel_setup=channel_setup,
            **kwargs
        )
        self.connection.pub_sub.subscribe(server_channels, self.connection)

    def unsubscribe(self, **kwargs):
        client_channel = kwargs.pop('channel')
        server_channels = make_channels(self.serializer_class, self.include_related, self.get_subscription_contexts(**kwargs))
        self.send(
            data='unsubscribed',
            channel_setup=self.make_channel_data(client_channel, server_channels),
            **kwargs)
        self.connection.pub_sub.unsubscribe(server_channels, self.connection)


class BaseModelPublisherRouter(BaseModelRouter):
    def publish_action(self, channels, data, action):
        publish_data = dict({'data': data})
        publish_data['action'] = action
        self.publish(channels, publish_data)

    def created(self, obj, **kwargs):
        super(BaseModelPublisherRouter, self).created(obj)
        base_channel = self.serializer_class.get_base_channel()
        all_model_channels = publisher.get_channels(base_channel)
        channels = filter_channels_by_model(all_model_channels, obj)
        self.publish_action(channels, self.serializer_class(instance=obj).serialize(), PUBACTIONS.created)

    def updated(self, obj, **kwargs):
        super(BaseModelPublisherRouter, self).updated(obj, **kwargs)
        base_channel = self.serializer_class.get_base_channel()
        all_model_channels = publisher.get_channels(base_channel)
        channels = filter_channels_by_model(all_model_channels, obj)
        updated_fields = kwargs.get('updated_fields')
        self.publish_action(channels, self.serializer.serialize(fields=updated_fields), PUBACTIONS.updated)
        past_state = kwargs.get('past_state')
        if past_state:
            previous_channels = filter_channels_by_dict(all_model_channels, past_state)
            delete_from_channels = set(previous_channels) - set(channels)
            self.publish_action(delete_from_channels, past_state, PUBACTIONS.deleted)

    def deleted(self, obj, obj_id, **kwargs):
        super(BaseModelPublisherRouter, self).deleted(obj, obj_id, **kwargs)
        base_channel = self.serializer_class.get_base_channel()
        all_model_channels = publisher.get_channels(base_channel)
        channels = filter_channels_by_model(all_model_channels, obj)
        data = self.serializer.serialize()
        self.publish_action(channels, data, PUBACTIONS.deleted)


class ModelRouter(BaseModelRouter):
    pass


class ModelPubRouter(BaseModelPublisherRouter):
    pass


def register(route):
    if route.get_name() in registered_handlers:
        return
    if issubclass(route, BaseModelRouter) or issubclass(route, BaseModelPublisherRouter):
        if 'get_single' in route.valid_verbs and 'get_single' not in route.exclude_verbs:
            if not hasattr(route, 'get_object'):
                raise Exception('\n-----------\nget_object needs to be implemented if get_single is available ({})\n-----------\n'.format(route.__name__))
        if 'get_list' in route.valid_verbs and 'get_list' not in route.exclude_verbs:
            if not hasattr(route, 'get_query_set'):
                raise Exception('\n-----------\nget_query_set needs to be implemented if get_list is available ({})\n-----------\n'.format(route.__name__))
    route_name = route.get_name()
    registered_handlers[route_name] = route


def get_route_handler(name):
    return registered_handlers[name]



from django.conf import settings
from ..cache import redis_cache_store, memory_cache_store

cache_store = None


def get_cache_store():
    if hasattr(settings, 'DRAGON_CACHE'):
        pass

    return memory_cache_store



from datetime import date
from decimal import Decimal
from django.db.models.fields.files import ImageFieldFile, FileField


class BaseSerializer(object):
    def serialize(self, value):
        return value


class DateSerializer(BaseSerializer):
    def serialize(self, value):
        return str(value)


class DecimalSerializer(BaseSerializer):
    def serialize(self, value):
        return str(value)


class FileSerializer(BaseSerializer):
    def serialize(self, value):
        try:
            return value.url
        except:
            return None


def serialize_field(value):
    if isinstance(value, date):
        return DateSerializer().serialize(value)
    if isinstance(value, Decimal):
        return DateSerializer().serialize(value)
    if isinstance(value, ImageFieldFile) or isinstance(value, FileField):
        return FileSerializer().serialize(value)
    return value



from django.db.models.signals import pre_delete, m2m_changed
from django.dispatch.dispatcher import receiver
from .pubsub_providers.base_provider import PUBACTIONS
from .model_tools import get_property, get_field_from_m2m_model_by_model
from .pubsub_providers.model_publisher import publish_model
from .serializers.serializer_importer import get_serializer


class SelfPublishModel(object):
    serializer_class = None

    def __init__(self, *args, **kwargs):
        if isinstance(self.serializer_class, str):
            self.serializer_class = get_serializer(self.serializer_class, self)
        self._pre_save_state = dict()
        super(SelfPublishModel, self).__init__(*args, **kwargs)
        self._serializer = self.serializer_class(instance=self)
        self._set_pre_save_state()

    def _set_pre_save_state(self):
        """
        Set the state of the model before any changes are done,
        so it's possible to determine what fields have changed.
        """
        relevant_fields = self._get_relevant_fields()

        for field in relevant_fields:
            val = get_property(self, field)
            if hasattr(self._serializer, field):
                continue
            if val is None:
                self._pre_save_state[field] = None
                continue
            self._pre_save_state[field] = val

    def _get_relevant_fields(self):
        """
        Get all fields that will affect the state.
        This is used to save the state of the model before it's updated,
        to be able to get changes used when publishing an update (so not all fields are published)
        """
        # update_fields = list(self._serializer.opts.update_fields)
        # publish_fields = list(self._serializer.opts.publish_fields)
        # relevant_fields = set(update_fields + publish_fields)
        relevant_fields = self._serializer.base_fields

        if 'id' in relevant_fields:
            relevant_fields.remove('id')

        return relevant_fields

    def get_changed_fields(self):
        changed_fields = []
        for k, v in self._pre_save_state.items():
            val = get_property(self, k)
            if val != v:
                changed_fields.append(k)
        return changed_fields

    def serialize(self):
        return self._serializer.serialize()

    def _publish(self, action, changed_fields=None):
        publish_model(self, self._serializer, action, changed_fields)

    def save(self, *args, **kwargs):
        if not self.pk:
            self.action = PUBACTIONS.created
            self.changed_fields = None
        else:
            self.action = PUBACTIONS.updated
            self.changed_fields = self.get_changed_fields()
        super(SelfPublishModel, self).save(*args, **kwargs)
        self._publish(self.action, self.changed_fields)


@receiver(m2m_changed)
def _self_publish_model_m2m_change(sender, instance, action, model, pk_set, **kwargs):
    if not isinstance(instance, SelfPublishModel):
        return
    instance.action = PUBACTIONS.updated
    if action in ['post_add', 'post_clear', 'post_remove']:
        instance._publish(instance.action, instance._serializer.opts.publish_fields)


@receiver(pre_delete)
def _self_publish_model_delete(sender, instance, **kwargs):
    if isinstance(instance, SelfPublishModel):
        instance._publish(PUBACTIONS.deleted)



from ..route_handler import BaseModelRouter, SUCCESS, ERROR
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelRouter):
    model = TwoFieldModel
    serializer_class = Serializer

    def get_object(self, **kwargs):
        return self.model.objects.get(pk=kwargs['id'])


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)
        self.obj = self.router.model.objects.create(text='text', number=1)

    def test_successful_delete(self):
        data = {'id': self.obj.pk}
        self.router.delete(**data)
        self.assertFalse(self.router.model.objects.exists())

    def test_deleted(self):
        data = {'id': self.obj.pk}
        self.router.delete(**data)
        actual = self.connection.last_message
        self.assertEqual(actual['context']['state'], SUCCESS)
        self.assertEqual(actual['data']['id'], data['id'])



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TextModel


class Serializer(ModelSerializer):
    class Meta:
        model = TextModel


class PaginatedRouter(BaseModelRouter):
    model = TextModel
    paginate_by = 5
    serializer_class = Serializer

    def get_query_set(self):
        return self.model.objects.all()


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = PaginatedRouter(self.connection)
        for i in range(21):
            TextModel.objects.create(text='text {}'.format(i))

    def test_pagination(self):
        result = self.router.get_list()
        self.assertEqual(len(result), PaginatedRouter.paginate_by)

    def test_get_second_page(self):
        self.router.context['page'] = 2
        result = self.router.get_list()
        self.assertNotEqual(result[0].pk, 1)

    def test_get_last_page(self):
        self.router.context['page'] = 5
        result = self.router.get_list()
        self.assertEqual(len(result), 1)



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TwoFieldModel


class FooSerializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel
        publish_fields = ('text', 'number')
        update_fields = ('text', )


class FooRouter(BaseModelRouter):
    route_name = 'foo'
    model = TwoFieldModel
    serializer_class = FooSerializer

    def get_initial(self, verb, **kwargs):
        initial = super(FooRouter, self).get_initial(verb, **kwargs)
        initial['number'] = 123
        return initial


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_get_initial_data(self):
        data = {'text': 'foo bar'}
        self.router.create(**data)
        object = TwoFieldModel.objects.get()
        self.assertEqual(object.number, 123)



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel


class Router(BaseModelRouter):
    model = TwoFieldModel
    serializer_class = Serializer


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)
        for i in range(5):
            TwoFieldModel.objects.create(text='text {}'.format(i), number=i)

    def test_send_list(self):
        qs = self.router.model.objects.all()
        self.router.send_list(qs)
        actual = self.connection.last_message['data']
        expected = [Serializer(instance=o).serialize() for o in qs]
        self.assertListEqual(actual, expected)



from swampdragon.serializers.model_serializer import ModelSerializer


class FooSelfPubSerializer(ModelSerializer):
    bars = 'BarSelfPubSerializer'

    class Meta:
        model = 'tests.FooSelfPub'
        publish_fields = ('name', 'number', 'bars')


class BarSelfPubSerializer(ModelSerializer):
    foo = FooSelfPubSerializer

    class Meta:
        model = 'tests.BarSelfPub'
        publish_fields = ('date', 'foo', 'id')



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel


class Router(BaseModelRouter):
    model = TwoFieldModel
    serializer_class = Serializer

    def get_subscription_contexts(self, **kwargs):
        context = super(Router, self).get_subscription_contexts(**kwargs)
        context['key'] = 'val'
        return context


class TestBaseModelRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)

    def test_get_subscription_context(self):
        actual = self.router.get_subscription_contexts()
        expected = {'key': 'val'}
        self.assertDictEqual(actual, expected)



class ModelValidationError(Exception):
    def __init__(self, errors={}, *args, **kwargs):
        super(ModelValidationError, self).__init__(*args, **kwargs)
        self.errors = errors

    def get_error_dict(self):
        return self.errors



from ..serializers.model_serializer import ModelSerializer
from ..route_handler import BaseModelRouter
from .dragon_test_case import DragonTestCase
from .models import TextModel


class FooModelSerializer(ModelSerializer):
    class Meta:
        model = TextModel
        publish_fields = ('text', )


class FooRouter(BaseModelRouter):
    route_name = 'foo'
    valid_verbs = ('get_list', )
    model = TextModel
    serializer_class = FooModelSerializer

    def get_query_set(self, **kwargs):
        return self.model.objects.all()


class TestBaseModelRouter(DragonTestCase):
    def test_get_list(self):
        TextModel.objects.create(text='foo bar')
        foos = FooRouter(self.connection).get_list()
        self.assertListEqual(list(foos), list(TextModel.objects.all()))



from django.db import models
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TextModel, SDModel


class FooOne2One(SDModel):
    name = models.CharField(max_length=20)


class BarOne2One(SDModel):
    foo = models.OneToOneField(FooOne2One)
    number = models.IntegerField()


class FooSerializer(ModelSerializer):
    bar = 'BarSerializer'

    class Meta:
        model = FooOne2One
        update_fields = ('name', 'bar')


class BarSerializer(ModelSerializer):
    foo = FooSerializer

    class Meta:
        model = BarOne2One
        update_fields = ('number', 'foo')


class TestModelSerializer(DragonTestCase):
    def test_deserialize_with_one_2_one(self):
        data = {
            'name': 'foo',
            'bar': {'number': 5}
        }
        serializer = FooSerializer(data)
        foo = serializer.save()
        self.assertEqual(foo.name, data['name'])
        self.assertEqual(foo.bar.number, data['bar']['number'])

    def test_deserialize_with_reverse_one_2_one(self):
        data = {
            'number': 123,
            'foo': {'name': 'foo'}
        }
        serializer = BarSerializer(data)
        bar = serializer.save()
        self.assertEqual(bar.number, data['number'])
        self.assertEqual(bar.foo.name, data['foo']['name'])
